@@ -113,25 +113,30 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M22<T> {
         }
     }
 
+    /// Calculate de QR factorization of the M22 via gram-schmidt
+    /// orthogonalization process
     fn qr(&self) -> Option<(Self, Self)> {
         let det = self.det();
         if det.abs() > T::epsilon() {
             let cols = self.get_cols();
-            let v2   = cols[1];
-            let mut u1 = cols[0];
-            let mut u2 = v2 - u1 * project_x_over_y(&*v2, &*u1);
-
-            normalize(&mut *u1);
-            normalize(&mut *u2);
-
-            let basis = V2::new([u1, u2]);
-            let q     = Self::new_from_vecs(basis);
+            let mut q: [V2<T>; 2] = *M22::zeros().get_cols();
+            for i in 0..q.len() {
+                let mut q_tilde = cols[i];
+                for k in 0..i {
+                    q_tilde -= q[k] * project_x_over_y(&*cols[i], &*q[k]);
+                }
+                normalize(&mut *q_tilde);
+                q[i] = q_tilde;
+            }
+            let basis = V2::new([q[0], q[1]]);
+            let q     = M22::new_from_vecs(basis);
             let r     = q.transpose() * (*self);
             Some((q, r))
         } else {
             None
         }
     }
+
 }
 
 impl<T: Num + Copy> M22<T> {
@@ -166,6 +171,22 @@ impl<T: Num + Copy> M22<T> {
         }
         result
     }
+
+    /// get the diagonal of the matrix
+    pub fn get_diagonal(&self) -> V2<T> {
+        let mut result = V2::zeros();
+        let mut index: usize = 0;
+        for i in 0..self.rows() {
+            for j in 0..self.cols() {
+                if i == j {
+                    result[index] = self[(i, j)];
+                    index += 1;
+                }
+            }
+        }
+        result
+    }
+
 }
 
 // M22 * V2
@@ -255,12 +276,12 @@ impl<T: Float + std::iter::Sum> M22<T> {
         let tau = self.trace();
         let delta = self.det();
         let tau_2 = tau * tau;
-        let four = T::from(4.0).unwrap();
+        let four = T::from(4)?;
         let discr = tau_2 - four * delta;
         if discr < T::zero() {
             None
         } else {
-            let two = T::from(2.0).unwrap();
+            let two = T::from(2)?;
             let lambda2 = (tau - T::sqrt(discr)) / two;
             let lambda1 = (tau + T::sqrt(discr)) / two;
             Some(V2::new([lambda1, lambda2]))
@@ -403,10 +424,10 @@ impl<T: Num + fmt::Display> fmt::Display for M22<T> {
 mod test_matrix2x2 {
     use crate::traits::LinearAlgebra;
     use crate::matrix2x2::M22;
-    use crate::utils::compare_vecs;
+    use crate::utils::{compare_vecs, nearly_equal};
     use crate::vector2::V2;
 
-    const EPS: f32 = 1e-8;
+    const EPS: f32 = 1e-7;
 
     #[test]
     fn create_m22_floats() {
@@ -581,4 +602,30 @@ mod test_matrix2x2 {
 
         assert!(compare_vecs(&result.as_vec(), &expected.as_vec(), EPS));
     }
+
+    #[test]
+    fn qr_test() {
+        let expected = m22_new!(10.0, 2.0;
+                                3.0, -4.0);
+        if let Some((q, r)) = expected.qr() {
+            let result = q * r;
+            assert!(compare_vecs(&result.as_vec(), &expected.as_vec(), EPS));
+            assert!(nearly_equal(q.det().abs(), 1.0, EPS));
+        }
+    }
+
+    #[test]
+    fn get_diagonal() {
+        let m = m22_new!(10.0, 2.0;
+                         3.0, -4.0);
+        let result = m.get_diagonal();
+        let expected = V2::new([10.0, -4.0]);
+        assert_eq!(
+            &result[..],
+            &expected[..],
+            "\nExpected\n{:?}\nfound\n{:?}",
+            &result[..],
+            &expected[..]
+        );
+    }
 }
