@@ -107,22 +107,23 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M33<T> {
         }
     }
 
+    /// Calculate de QR factorization of the M33 via gram-schmidt
+    /// orthogonalization process
     fn qr(&self) -> Option<(Self, Self)> {
         let det = self.det();
         if det.abs() > T::epsilon() {
             let cols = self.get_cols();
-            let v2 = cols[1];
-            let v3 = cols[2];
-
-            let mut u1 = cols[0];
-            let mut u2 = v2 - u1 * project_x_over_y(&*v2, &*u1);
-            let mut u3 = v3 - u1 * project_x_over_y(&*v3, &*u1) - u2 * project_x_over_y(&*v3, &*u2);
-            normalize(&mut *u1);
-            normalize(&mut *u2);
-            normalize(&mut *u3);
-
-            let basis = V3::new([u1, u2, u3]);
-            let q     = Self::new_from_vecs(basis);
+            let mut q: [V3<T>; 3] = *M33::zeros().get_cols();
+            for i in 0..q.len() {
+                let mut q_tilde = cols[i];
+                for k in 0..i {
+                    q_tilde -= q[k] * project_x_over_y(&*cols[i], &*q[k]);
+                }
+                normalize(&mut *q_tilde);
+                q[i] = q_tilde;
+            }
+            let basis = V3::new([q[0], q[1], q[2]]);
+            let q     = M33::new_from_vecs(basis);
             let r     = q.transpose() * (*self);
             Some((q, r))
         } else {
@@ -178,6 +179,52 @@ impl<T: Num + Copy> M33<T> {
         }
         result
     }
+
+    /// get the diagonal of the matrix
+    pub fn get_diagonal(&self) -> V3<T> {
+        let mut result = V3::zeros();
+        let mut index: usize = 0;
+        for i in 0..self.rows() {
+            for j in 0..self.cols() {
+                if i == j {
+                    result[index] = self[(i, j)];
+                    index += 1;
+                }
+            }
+        }
+        result
+    }
+
+    pub fn get_upper_triagular(&self) -> [T; 3] {
+        let zero = T::zero();
+        let mut result: [T; 3] = [zero, zero, zero];
+        let mut index = 0;
+        for i in 0..self.rows() {
+            for j in 0..self.cols() {
+                if i < j {
+                    result[index] = self[(i, j)];
+                    index += 1;
+                }
+            }
+        }
+        result
+    }
+
+    pub fn get_lower_triangular(&self) -> [T; 3] {
+        let zero = T::zero();
+        let mut result: [T; 3] = [zero, zero, zero];
+        let mut index = 0;
+        for i in 0..self.rows() {
+            for j in 0..self.cols() {
+                if i > j {
+                    result[index] = self[(i, j)];
+                    index += 1;
+                }
+            }
+        }
+        result
+    }
+
 }
 
 impl<T: Num + Copy> M33<T> {
@@ -625,4 +672,64 @@ mod test_matrix3x3 {
         assert!(compare_vecs(&result.as_vec(), &expected.as_vec(), EPS));
     }
 
+    #[test]
+    fn qr_test() {
+        let expected = m33_new!(0.0, 1.0, 2.0;
+                                3.0, 4.0, 5.0;
+                                6.0, 7.0, 8.0);
+        if let Some((q, r)) = expected.qr() {
+            let result = q * r;
+            assert!(compare_vecs(&result.as_vec(), &expected.as_vec(), EPS));
+            assert!(nearly_equal(q.det().abs(), 1.0, EPS));
+        }
+    }
+
+    #[test]
+    fn get_diagonal() {
+        let m = m33_new!(0.0, 1.0, 2.0;
+                         3.0, 4.0, 5.0;
+                         6.0, 7.0, 8.0);
+        let result = m.get_diagonal();
+        let expected = V3::new([0.0, 4.0, 8.0]);
+        assert_eq!(
+            &result[..],
+            &expected[..],
+            "\nExpected\n{:?}\nfound\n{:?}",
+            &result[..],
+            &expected[..]
+        );
+    }
+
+    #[test]
+    fn get_upper_triangular_test() {
+        let m = m33_new!(0.0, 1.0, 2.0;
+                         3.0, 4.0, 5.0;
+                         6.0, 7.0, 8.0);
+        let result = m.get_upper_triagular();
+        let expected = [1.0, 2.0, 5.0];
+        assert_eq!(
+            &result[..],
+            &expected[..],
+            "\nExpected\n{:?}\nfound\n{:?}",
+            &result[..],
+            &expected[..]
+        );
+    }
+
+    #[test]
+    fn get_lower_triangular_test() {
+        let m = m33_new!(0.0, 1.0, 2.0;
+                         3.0, 4.0, 5.0;
+                         6.0, 7.0, 8.0);
+        let result = m.get_lower_triangular();
+        let expected = [3.0, 6.0, 7.0];
+        assert_eq!(
+            &result[..],
+            &expected[..],
+            "\nExpected\n{:?}\nfound\n{:?}",
+            &result[..],
+            &expected[..]
+        );
+    }
+
 }
