@@ -198,26 +198,22 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M44<T> {
         }
     }
 
+    /// Calculate de QR factorization of the M44 via gram-schmidt
+    /// orthogonalization process
     fn qr(&self) -> Option<(Self, Self)> {
         let det = self.det();
         if det.abs() > T::epsilon() {
             let cols = self.get_cols();
-            // this are the columns of the matrix
-            let v2 = cols[1];
-            let v3 = cols[2];
-            let v4 = cols[3];
-
-            let mut u1 = cols[0];
-            let mut u2 = v2 - u1 * project_x_over_y(&*v2, &*u1);
-            let mut u3 = v3 - u1 * project_x_over_y(&*v3, &*u1) - u2 * project_x_over_y(&*v3, &*u2);
-            let mut u4 = v4 - u1 * project_x_over_y(&*v4, &*u1) - u2 * project_x_over_y(&*v4, &*u2) - u3 * project_x_over_y(&*v4, &*u3);
-
-            normalize(&mut *u1);
-            normalize(&mut *u2);
-            normalize(&mut *u3);
-            normalize(&mut *u4);
-
-            let basis = V4::new([u1, u2, u3, u4]);
+            let mut q: [V4<T>; 4] = *M44::zeros().get_cols();
+            for i in 0..q.len() {
+                let mut q_tilde = cols[i];
+                for k in 0..i {
+                    q_tilde -= q[k] * project_x_over_y(&*cols[i], &*q[k]);
+                }
+                normalize(&mut *q_tilde);
+                q[i] = q_tilde;
+            }
+            let basis = V4::new([q[0], q[1], q[2], q[3]]);
             let q     = M44::new_from_vecs(basis);
             let r     = q.transpose() * (*self);
             Some((q, r))
@@ -236,7 +232,7 @@ impl<T> M44<T> {
     pub fn rows(&self) -> usize {
         self.0.len()
     }
-    // NOTE(elsuizo:2019-09-13): si ya se es medio...
+
     pub fn cols(&self) -> usize {
         self.rows()
     }
@@ -276,6 +272,20 @@ impl<T: Num + Copy> M44<T> {
         result
     }
 
+    /// get the diagonal of the matrix
+    pub fn get_diagonal(&self) -> V4<T> {
+        let mut result = V4::zeros();
+        let mut index: usize = 0;
+        for i in 0..self.rows() {
+            for j in 0..self.cols() {
+                if i == j {
+                    result[index] = self[(i, j)];
+                    index += 1;
+                }
+            }
+        }
+        result
+    }
     // TODO(elsuizo:2020-06-02): this could be optimize
 
     pub fn get_submatrix(&self, selected: (usize, usize)) -> M33<T> {
@@ -330,6 +340,37 @@ impl<T: Num + Copy> M44<T> {
         }
         V4::new([c0, c1, c2, c3])
     }
+
+    pub fn get_upper_triagular(&self) -> [T; 6] {
+        let zero = T::zero();
+        let mut result: [T; 6] = [zero; 6];
+        let mut index = 0;
+        for i in 0..self.rows() {
+            for j in 0..self.cols() {
+                if i < j {
+                    result[index] = self[(i, j)];
+                    index += 1;
+                }
+            }
+        }
+        result
+    }
+
+    pub fn get_lower_triangular(&self) -> [T; 6] {
+        let zero = T::zero();
+        let mut result: [T; 6] = [zero; 6];
+        let mut index = 0;
+        for i in 0..self.rows() {
+            for j in 0..self.cols() {
+                if i > j {
+                    result[index] = self[(i, j)];
+                    index += 1;
+                }
+            }
+        }
+        result
+    }
+
 }
 
 impl<T: Num + Copy> Add for M44<T> {
@@ -974,9 +1015,9 @@ mod test_matrix4x4 {
     #[test]
     fn new_from_vecs_test() {
         let expected = m44_new!( 0.0,  1.0,  2.0,  3.0;
-                          4.0,  5.0,  6.0,  7.0;
-                          8.0,  9.0, 10.0, 11.0;
-                         12.0, 13.0, 14.0, 15.0);
+                                 4.0,  5.0,  6.0,  7.0;
+                                 8.0,  9.0, 10.0, 11.0;
+                                12.0, 13.0, 14.0, 15.0);
 
         let cols = expected.get_cols();
 
@@ -984,4 +1025,69 @@ mod test_matrix4x4 {
 
         assert!(compare_vecs(&result.as_vec(), &expected.as_vec(), EPS));
     }
+
+    #[test]
+    fn qr_test() {
+        let expected = m44_new!( 0.0,  1.0,  2.0,  3.0;
+                                 4.0,  5.0,  6.0,  7.0;
+                                 8.0,  9.0, 10.0, 11.0;
+                                12.0, 13.0, 14.0, 15.0);
+        if let Some((q, r)) = expected.qr() {
+            let result = q * r;
+            assert!(compare_vecs(&result.as_vec(), &expected.as_vec(), EPS));
+            assert!(nearly_equal(q.det().abs(), 1.0, EPS));
+        }
+    }
+
+    #[test]
+    fn get_diagonal() {
+        let m = m44_new!( 0.0,  1.0,  2.0,  3.0;
+                          4.0,  5.0,  6.0,  7.0;
+                          8.0,  9.0, 10.0, 11.0;
+                         12.0, 13.0, 14.0, 15.0);
+        let result = m.get_diagonal();
+        let expected = V4::new([0.0, 5.0, 10.0, 15.0]);
+        assert_eq!(
+            &result[..],
+            &expected[..],
+            "\nExpected\n{:?}\nfound\n{:?}",
+            &result[..],
+            &expected[..]
+        );
+    }
+
+    #[test]
+    fn get_upper_triangular_test() {
+        let m = m44_new!( 0.0,  1.0,  2.0,  3.0;
+                          4.0,  5.0,  6.0,  7.0;
+                          8.0,  9.0, 10.0, 11.0;
+                         12.0, 13.0, 14.0, 15.0);
+        let result = m.get_upper_triagular();
+        let expected = [1.0, 2.0, 3.0, 6.0, 7.0, 11.0];
+        assert_eq!(
+            &result[..],
+            &expected[..],
+            "\nExpected\n{:?}\nfound\n{:?}",
+            &result[..],
+            &expected[..]
+        );
+    }
+
+    #[test]
+    fn get_lower_triangular_test() {
+        let m = m44_new!( 0.0,  1.0,  2.0,  3.0;
+                          4.0,  5.0,  6.0,  7.0;
+                          8.0,  9.0, 10.0, 11.0;
+                         12.0, 13.0, 14.0, 15.0);
+
+        let result = m.get_lower_triangular();
+        let expected = [4.0, 8.0, 9.0, 12.0, 13.0, 14.0];
+        assert_eq!(
+            &result[..],
+            &expected[..],
+            "\nExpected\n{:?}\nfound\n{:?}",
+            &result[..],
+            &expected[..]
+        );
+    }
 }
