@@ -46,14 +46,17 @@ use crate::traits::LinearAlgebra;
 pub struct M66<T>([[T; 6]; 6]);
 
 impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M66<T> {
+    #[inline]
     fn rows(&self) -> usize {
         self.0.len()
     }
 
+    #[inline]
     fn cols(&self) -> usize {
         self.rows()
     }
 
+    #[inline]
     fn transpose(&self) -> Self {
         let a_00 = self[(0, 0)];
         let a_01 = self[(0, 1)];
@@ -102,6 +105,7 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M66<T> {
         ])
     }
 
+    #[inline]
     fn trace(&self) -> T {
         return self[(0, 0)]
             + self[(1, 1)]
@@ -905,33 +909,24 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M66<T> {
         }
     }
 
+    /// Calculate de QR factorization of the M66 via gram-schmidt
+    /// orthogonalization process
     fn qr(&self) -> Option<(Self, Self)> {
         let det = self.det();
         if det.abs() > T::epsilon() {
             let cols = self.get_cols();
-            // this are the columns of the matrix
-            let v2 = cols[1];
-            let v3 = cols[2];
-            let v4 = cols[3];
-            let v5 = cols[4];
-            let v6 = cols[5];
-
-            let mut u1 = cols[0];
-            let mut u2 = v2 - u1 * project_x_over_y(&*v2, &*u1);
-            let mut u3 = v3 - u1 * project_x_over_y(&*v3, &*u1) - u2 * project_x_over_y(&*v3, &*u2);
-            let mut u4 = v4 - u1 * project_x_over_y(&*v4, &*u1) - u2 * project_x_over_y(&*v4, &*u2) - u3 * project_x_over_y(&*v4, &*u3);
-            let mut u5 = v5 - u1 * project_x_over_y(&*v5, &*u1) - u2 * project_x_over_y(&*v5, &*u2) - u3 * project_x_over_y(&*v5, &*u3) - u4 * project_x_over_y(&*v5, &*u4);
-            let mut u6 = v6 - u1 * project_x_over_y(&*v6, &*u1) - u2 * project_x_over_y(&*v6, &*u2) - u3 * project_x_over_y(&*v6, &*u3) - u4 * project_x_over_y(&*v6, &*u4) - u5 * project_x_over_y(&*v6, &*u5);
-
-            normalize(&mut *u1);
-            normalize(&mut *u2);
-            normalize(&mut *u3);
-            normalize(&mut *u4);
-            normalize(&mut *u5);
-            normalize(&mut *u6);
-
-            let basis = V6::new([u1, u2, u3, u4, u5, u6]);
-            let q     = Self::new_from_vecs(basis);
+            let mut q: [V6<T>; 6] = *M66::zeros().get_cols();
+            for i in 0..q.len() {
+                let mut q_tilde = cols[i];
+                for k in 0..i {
+                    q_tilde -= q[k] * project_x_over_y(&*cols[i], &*q[k]);
+                }
+                normalize(&mut *q_tilde);
+                q[i] = q_tilde;
+            }
+            // TODO(elsuizo:2020-08-05): do this with a another for loop
+            let basis = V6::new([q[0], q[1], q[2], q[3], q[4], q[5]]);
+            let q     = M66::new_from_vecs(basis);
             let r     = q.transpose() * (*self);
             Some((q, r))
         } else {
@@ -1440,6 +1435,21 @@ impl<T: Num + Copy + std::iter::Sum> M66<T> {
         result
     }
 
+    /// get the diagonal of the matrix
+    pub fn get_diagonal(&self) -> V6<T> {
+        let mut result = V6::zeros();
+        let mut index: usize = 0;
+        for i in 0..self.rows() {
+            for j in 0..self.cols() {
+                if i == j {
+                    result[index] = self[(i, j)];
+                    index += 1;
+                }
+            }
+        }
+        result
+    }
+
     pub fn new_from_vecs(cols: V6<V6<T>>) -> Self {
         let mut result = Self::zeros();
 
@@ -1518,6 +1528,36 @@ impl<T: Num + Copy + std::iter::Sum> M66<T> {
         }
         V6::new([c0, c1, c2, c3, c4, c5])
     }
+
+    pub fn get_upper_triangular(&self) -> [T; 15] {
+        let zero = T::zero();
+        let mut result: [T; 15] = [zero; 15];
+        let mut index = 0;
+        for i in 0..self.rows() {
+            for j in 0..self.cols() {
+                if i < j {
+                    result[index] = self[(i, j)];
+                    index += 1;
+                }
+            }
+        }
+        result
+    }
+
+    pub fn get_lower_triangular(&self) -> [T; 15] {
+        let zero = T::zero();
+        let mut result: [T; 15] = [zero; 15];
+        let mut index = 0;
+        for i in 0..self.rows() {
+            for j in 0..self.cols() {
+                if i > j {
+                    result[index] = self[(i, j)];
+                    index += 1;
+                }
+            }
+        }
+        result
+    }
 }
 
 impl<T: Num + Copy> Zero for M66<T> {
@@ -1922,4 +1962,76 @@ mod test_matrix6x6 {
 
         assert!(compare_vecs(&result.as_vec(), &expected.as_vec(), EPS));
     }
+
+    #[test]
+    fn qr_test() {
+        let expected = m66_new!( 1.0,  1.0, 3.0,  4.0,  9.0, 3.0;
+                                10.0, 10.0, 1.0,  2.0,  2.0, 5.0;
+                                 2.0,  9.0, 6.0, 10.0, 10.0, 9.0;
+                                10.0,  9.0, 9.0,  7.0,  3.0, 6.0;
+                                 7.0,  6.0, 6.0,  2.0,  9.0, 5.0;
+                                 3.0,  8.0, 1.0,  4.0,  1.0, 5.0);
+        if let Some((q, r)) = expected.qr() {
+            let result = q * r;
+            assert!(compare_vecs(&result.as_vec(), &expected.as_vec(), EPS));
+            assert!(nearly_equal(q.det().abs(), 1.0, EPS));
+        }
+    }
+
+    #[test]
+    fn get_diagonal_test() {
+        let m = m66_new!( 1.0,  1.0, 3.0,  4.0,  9.0, 3.0;
+                         10.0, 10.0, 1.0,  2.0,  2.0, 5.0;
+                          2.0,  9.0, 6.0, 10.0, 10.0, 9.0;
+                         10.0,  9.0, 9.0,  7.0,  3.0, 6.0;
+                          7.0,  6.0, 6.0,  2.0,  9.0, 5.0;
+                          3.0,  8.0, 1.0,  4.0,  1.0, 5.0);
+        let result = m.get_diagonal();
+        let expected = V6::new([1.0, 10.0, 6.0, 7.0, 9.0, 5.0]);
+        assert_eq!(
+            &result[..],
+            &expected[..],
+            "\nExpected\n{:?}\nfound\n{:?}",
+            &result[..],
+            &expected[..]
+        );
+    }
+
+    #[test]
+    fn get_upper_triangular_test() {
+        let m = m66_new!( 1.0,  1.0, 3.0,  4.0,  9.0, 3.0;
+                         10.0, 10.0, 1.0,  2.0,  2.0, 5.0;
+                          2.0,  9.0, 6.0, 10.0, 10.0, 9.0;
+                         10.0,  9.0, 9.0,  7.0,  3.0, 6.0;
+                          7.0,  6.0, 6.0,  2.0,  9.0, 5.0;
+                          3.0,  8.0, 1.0,  4.0,  1.0, 5.0);
+        let result = m.get_upper_triangular();
+        let expected = [1.0, 3.0, 4.0, 9.0, 3.0, 1.0, 2.0, 2.0, 5.0, 10.0, 10.0, 9.0, 3.0, 6.0, 5.0];
+        assert_eq!(
+            &result[..],
+            &expected[..],
+            "\nExpected\n{:?}\nfound\n{:?}",
+            &result[..],
+            &expected[..]
+        );
+    }
+
+    #[test]
+    fn get_lower_triangular_test() {
+        let m = m66_new!( 1.0,  1.0, 3.0,  4.0,  9.0, 3.0;
+                         10.0, 10.0, 1.0,  2.0,  2.0, 5.0;
+                          2.0,  9.0, 6.0, 10.0, 10.0, 9.0;
+                         10.0,  9.0, 9.0,  7.0,  3.0, 6.0;
+                          7.0,  6.0, 6.0,  2.0,  9.0, 5.0;
+                          3.0,  8.0, 1.0,  4.0,  1.0, 5.0);
+        let result = m.get_lower_triangular();
+        let expected = [10.0, 2.0, 9.0, 10.0, 9.0, 9.0, 7.0, 6.0, 6.0, 2.0, 3.0, 8.0, 1.0, 4.0, 1.0];
+        assert_eq!(
+            &result[..],
+            &expected[..],
+            "\nExpected\n{:?}\nfound\n{:?}",
+            &result[..],
+            &expected[..]
+        );
+    }
 }
