@@ -29,6 +29,7 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //-------------------------------------------------------------------------
 use num::{Num, Float};
+use crate::utils::compare_floats;
 
 #[derive(Copy, Clone, Debug, PartialEq)]
 pub struct MaxMin<T> {
@@ -62,9 +63,9 @@ pub fn dot<T: Num + Copy + std::iter::Sum>(slice1: &[T], slice2: &[T]) -> T {
 /// normalize the slice
 pub fn normalize<T: Float>(slice: &mut [T]) {
     let n = norm2(slice);
-    for element in slice.iter_mut() {
+    slice.iter_mut().for_each(|element| {
         *element = *element / n;
-    }
+    })
 }
 
 /// project x in the direction of y
@@ -72,6 +73,14 @@ pub fn project_x_over_y<T: Float + std::iter::Sum>(x: &[T], y: &[T]) -> T {
     dot(x, y) / dot(y, y)
 }
 
+pub fn check_elements<T: Float>(v: &[T], tol: T) -> bool {
+    let mut result = false;
+    for num in v.iter() {
+        result |= compare_floats(*num, T::zero(), tol);
+    }
+    result
+}
+
 //-------------------------------------------------------------------------
 //                        tests
 //-------------------------------------------------------------------------
@@ -92,4 +101,31 @@ mod test_slides_methods {
         assert_eq!(result, expected);
 
     }
+
+    #[test]
+    fn dot_tests() {
+       let v1 = V3::new([1, 1, 1]);
+       let v2 = V3::new([1, 1, 3]);
+
+       let result = dot(&*v1, &*v2);
+       let expected = 5;
+
+       assert_eq!(result, expected);
+    }
+
+    #[test]
+    fn normalize_test() {
+        let mut v1 = V3::new([1.0, 1.0, 1.0]);
+        normalize(&mut *v1);
+
+        let expected = V3::new([0.5773502691896258, 0.5773502691896258, 0.5773502691896258]);
+
+        assert_eq!(
+            &v1[..],
+            &expected[..],
+            "\nExpected\n{:?}\nfound\n{:?}",
+            &v1[..],
+            &expected[..]
+        );
+    }
 }
